/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : SCPTestDefinition.cc                                     */ 
/*                                                                       */ 
/*       Date : November 20, 1998                                        */
/* Updated at : November 20, 1998                                        */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>

#include "SCPTestDefinition.h"

/*************************************************************************/
/*************************************************************************/
SCPTestDefinition::SCPTestDefinition()
{

     dataFileName = new char[50];
     rfileName    = new char[50];
     brfileName   = new char[50];
     sfileName    = new char[50];
     repfileName    = new char[50];

     sprintf(dataFileName,"data");
     sprintf(rfileName,"results.out");
     sprintf(brfileName,"bresults.out");
     sprintf(sfileName,"stats.out");
     sprintf(repfileName,"report.out");

     cProb = 0.5;
     mProb = 0.2;
     numberOfGenerations = 1;
     sizeOfPopulation = 5;

     cType = RND1PX;
     mType = DPXRND;
     aType = RANDOM_ALG;
     ifType = BINARY;

}// end of CONSTRUCTOR

/*************************************************************************/
/*************************************************************************/
SCPTestDefinition::SCPTestDefinition(const SCPTestDefinition& RHS)
{ copyFrom(RHS); }

/*************************************************************************/
/*************************************************************************/
void 
SCPTestDefinition::copyFrom(const SCPTestDefinition& RHS)
{ 

     dataFileName = new char[strlen(RHS.dataFileName)+1];
     sprintf(dataFileName,RHS.dataFileName);

     rfileName = new char[strlen(RHS.rfileName)+1];
     sprintf(rfileName,RHS.rfileName);

     brfileName = new char[strlen(RHS.brfileName)+1];
     sprintf(brfileName,RHS.brfileName);

     sfileName = new char[strlen(RHS.sfileName)+1];
     sprintf(sfileName,RHS.sfileName);

     repfileName = new char[strlen(RHS.repfileName)+1];
     sprintf(repfileName,RHS.repfileName);

     cProb     = RHS.cProb;
     mProb     = RHS.mProb;

     numberOfGenerations  = RHS.numberOfGenerations;
     sizeOfPopulation     = RHS.sizeOfPopulation;

     aType = RHS.aType;
     cType = RHS.cType;
     mType = RHS.mType;
     ifType = RHS.ifType;

}// end of copyFrom()

/*************************************************************************/
/*************************************************************************/
char* SCPTestDefinition::getDataFileName()
{  return dataFileName; }

/*************************************************************************/
/*************************************************************************/
void   
SCPTestDefinition::setDataFileName(char *newName)
{
   dataFileName = new char[strlen(newName)+1];
   sprintf(dataFileName,"%s",newName);

}// end of setDataFileName()

/*************************************************************************/
/** All results will be written to this file */
/*************************************************************************/
char* SCPTestDefinition::getResultFileName() {  return rfileName; }

/*************************************************************************/
/** All results will be written to this file */
/*************************************************************************/
void   
SCPTestDefinition::setResultFileName(char *newName)
{
   rfileName = new char[strlen(newName)+1];
   sprintf(rfileName,"%s",newName);

}// end of setResultFileName()

/*************************************************************************/
/** The best result will be written to this file */
/*************************************************************************/
char* SCPTestDefinition::getBinResultFileName() {  return brfileName; }

/*************************************************************************/
/** The best results will be written to this file */
/*************************************************************************/
void SCPTestDefinition::setBinResultFileName(char *newName)
{
   brfileName = new char[strlen(newName)+1];
   sprintf(brfileName,"%s",newName);

}// end of setBinResultFileName()

/*************************************************************************/
/** All stats will be written to this file */
/*************************************************************************/
char* SCPTestDefinition::getStatFileName() {  return sfileName; }

/*************************************************************************/
/** All stats will be written to this file */
/*************************************************************************/
void SCPTestDefinition::setStatFileName(char *newName)
{
   sfileName = new char[strlen(newName)+1];
   sprintf(sfileName,"%s",newName);

}// end of setStatFileName()

/*************************************************************************/
/** report will be written to this file */
/*************************************************************************/
char* SCPTestDefinition::getReportFileName() {  return repfileName; }

/*************************************************************************/
/** report will be written to this file */
/*************************************************************************/
void SCPTestDefinition::setReportFileName(char *newName)
{
   repfileName = new char[strlen(newName)+1];
   sprintf(repfileName,"%s",newName);

}// end of setReportFileName()

/*************************************************************************/
/*************************************************************************/
InputFileType SCPTestDefinition::getInputFileType() {  return ifType; }

/*************************************************************************/
/*************************************************************************/
void SCPTestDefinition::setInputFileType(InputFileType _ifType) 
{  ifType=_ifType; }

/*************************************************************************/
/*************************************************************************/
AlgorithmType SCPTestDefinition::getAlgorithmType() { return aType; }

/*************************************************************************/
/*************************************************************************/
void SCPTestDefinition::setAlgorithmType(AlgorithmType _aType) { aType=_aType; }

/*************************************************************************/
/*************************************************************************/
CrossoverOprType SCPTestDefinition::getCrossoverOprType() {  return cType; }

/*************************************************************************/
/*************************************************************************/
void SCPTestDefinition::setCrossoverOprType(CrossoverOprType _cType) 
{  cType=_cType; }

/*************************************************************************/
/*************************************************************************/
float SCPTestDefinition::getCrossProbability() {  return cProb; }

/*************************************************************************/
/*************************************************************************/
void SCPTestDefinition::setCrossProbability(float val) {  cProb=val; }

/*************************************************************************/
/*************************************************************************/
MutationOprType SCPTestDefinition::getMutationOprType() {  return mType; }

/*************************************************************************/
/*************************************************************************/
void SCPTestDefinition::setMutationOprType(MutationOprType _mType) 
{ mType=_mType; }

/*************************************************************************/
/*************************************************************************/
float SCPTestDefinition::getMutationProbability() {  return mProb; }

/*************************************************************************/
/*************************************************************************/
void SCPTestDefinition::setMutationProbability(float val) {  mProb=val; }

/*************************************************************************/
/*************************************************************************/
SCPTestDefinition::getNumberOfGenerations() {  return numberOfGenerations; }

/*************************************************************************/
/*************************************************************************/
SCPTestDefinition::getPopulationSize() {  return sizeOfPopulation; }

/*************************************************************************/
/*************************************************************************/
SCPTestDefinition SCPTestDefinition::operator=(const SCPTestDefinition& RHS)
{  
   copyFrom(RHS); 
   return *this; 
}// end of operator=()

/*************************************************************************/
/**
  Reads the CONFIGURATION file and returns a SCPTestDefinition object.

  fname data
  iftype [ BINARY | CODED | ORLIB]
  rfname resultfn
  brfname bresultfn
  sfname statfn
  repfname report
  cprob 0.5
  mprob 0.2
  gennum 1
  spop  5
  mvaries     [yes|no]
  collectstat [on|off]
  esampling   [on|off]
  copr        [ 1PXRND | 1PXCHV | 1PXBCK | 1PXGR3 | 2PXRND | 2PXCHV | 
		2PXBCK | 2PXGR3 | UNIRND | UNICHV | UNIBCK | UNIGR3 |
		FUSION ]
  mopr        [ DPXRND | DPXCHV | DPXBCK | DPXGR3 |
		WDLRND | WDLCHV | WDLBCK | WDLGR3 ]
  algorithm   [ GREEDY | GA | RND | BACK | CHAVATAL | GREEDY3 | BCGA | SSGA ]

*/
/*************************************************************************/
SCPTestDefinition 
SCPTestDefinition::readTestDefinition(char* fileName)
{
   char kWord[50], value[50];
   int  i;

   SCPTestDefinition ret;

   ifstream fileIn(fileName);

   while(!fileIn.eof())
   {
      fileIn >> kWord >> value ;

      if(!fileIn.eof())
      {
	 if(!strcmp(kWord,"fname")) ret.setDataFileName(value);
	 else if(!strcmp(kWord,"rfname")) 
	      ret.setResultFileName(value);
	 else if( !strcmp(kWord,"iftype") )
	      { 
		if(!strcmp(value,"CODED")) ret.setInputFileType(CODED);
		else if(!strcmp(value,"ORLIB")) ret.setInputFileType(ORLIB);
		else if(!strcmp(value,"BINARY")) ret.setInputFileType(BINARY);
	      }
	 else if(!strcmp(kWord,"brfname")) 
	      ret.setBinResultFileName(value);
	 else if(!strcmp(kWord,"sfname")) 
	      ret.setStatFileName(value);
	 else if(!strcmp(kWord,"repfname")) 
	      ret.setReportFileName(value);
	 else if(!strcmp(kWord,"cprob")) 
	      ret.setCrossProbability(atof(value));
	 else if(!strcmp(kWord,"mprob"))
	      ret.setMutationProbability(atof(value));
	 else if(!strcmp(kWord,"gennum"))
	      ret.setNumberOfGenerations(atoi(value));
	 else if(!strcmp(kWord,"spop"))
	      ret.setPopulationSize(atoi(value));
	 else if(!strcmp(kWord,"copr"))
	      {

	             if(!strcmp(value,"1PXRND")) 
		     ret.setCrossoverOprType(RND1PX);
	        else if(!strcmp(value,"1PXCHV"))
	             ret.setCrossoverOprType(CHV1PX);
	        else if(!strcmp(value,"1PXBCK"))
	             ret.setCrossoverOprType(BCK1PX);
	        else if(!strcmp(value,"1PXGR3"))
	             ret.setCrossoverOprType(GR31PX);

                else if(!strcmp(value,"2PXRND")) 
		     ret.setCrossoverOprType(RND2PX);
	        else if(!strcmp(value,"2PXCHV"))
	             ret.setCrossoverOprType(CHV2PX);
	        else if(!strcmp(value,"2PXBCK"))
	             ret.setCrossoverOprType(BCK2PX);
	        else if(!strcmp(value,"2PXGR3"))
	             ret.setCrossoverOprType(GR32PX);

                else if(!strcmp(value,"DPXRND")) 
		     ret.setCrossoverOprType(XDPRND);
	        else if(!strcmp(value,"DPXCHV"))
	             ret.setCrossoverOprType(XDPCHV);
	        else if(!strcmp(value,"DPXBCK"))
	             ret.setCrossoverOprType(XDPBCK);
	        else if(!strcmp(value,"DPXGR3"))
	             ret.setCrossoverOprType(XDPGR3);

                else if(!strcmp(value,"UNIRND")) 
		     ret.setCrossoverOprType(UNIRND);
	        else if(!strcmp(value,"UNICHV"))
	             ret.setCrossoverOprType(UNICHV);
	        else if(!strcmp(value,"UNIBCK"))
	             ret.setCrossoverOprType(UNIBCK);
	        else if(!strcmp(value,"UNIGR3"))
	             ret.setCrossoverOprType(UNIGR3);
	        else if(!strcmp(value,"FUSION"))
	             ret.setCrossoverOprType(FUSION);
	      }

	 else if(!strcmp(kWord,"mopr"))
	      {

	             if(!strcmp(value,"DPXRND")) 
		     ret.setMutationOprType(DPXRND);
	        else if(!strcmp(value,"DPXCHV"))
	             ret.setMutationOprType(DPXCHV);
	        else if(!strcmp(value,"DPXBCK"))
	             ret.setMutationOprType(DPXBCK);
	        else if(!strcmp(value,"DPXGR3"))
	             ret.setMutationOprType(DPXGR3);
	        else if(!strcmp(value,"WDLRND"))
	             ret.setMutationOprType(WDLRND);
	        else if(!strcmp(value,"WDLCHV"))
	             ret.setMutationOprType(WDLCHV);
	        else if(!strcmp(value,"WDLBCK"))
	             ret.setMutationOprType(WDLBCK);
	        else if(!strcmp(value,"WDLGR3"))
	             ret.setMutationOprType(WDLGR3);

	      }

	 else if(!strcmp(kWord,"algorithm"))
	      {
	             if(!strcmp(value,"GREEDY")) 
		     ret.setAlgorithmType(GREEDY);
	        else if(!strcmp(value,"GA")) ret.setAlgorithmType(GA);
	        else if(!strcmp(value,"RANDOM"))
	             ret.setAlgorithmType(RANDOM_ALG);
	        else if(!strcmp(value,"BACK")) ret.setAlgorithmType(BACK);
	        else if(!strcmp(value,"CHAVATAL")) ret.setAlgorithmType(CHV);
	        else if(!strcmp(value,"GREEDY3")) ret.setAlgorithmType(GRD3);
	        else if(!strcmp(value,"BCGA")) ret.setAlgorithmType(BCGA);
	        else if(!strcmp(value,"SSGA")) ret.setAlgorithmType(SSGA);
	      }

      }// end of if 

      for(i=0;i<50;i++) kWord[i]=value[i]=NULL;

   }// end of while

   fileIn.close();

   return ret;

}// end of readTestDefinition()

/*************************************************************************/
/*************************************************************************/
ostream& operator<<(ostream& os, const SCPTestDefinition& RHS)
{
   int i=0;

   os << " File Name .............: " << RHS.dataFileName << endl;
   os << " Results File Name .....: " << RHS.rfileName << endl;
   os << " Bin Result File Name ..: " << RHS.brfileName << endl;
   os << " Stat File Name ........: " << RHS.sfileName << endl;
   os << " Report File Name ......: " << RHS.repfileName << endl;
   os << " Algorithm Type ........: ";
   
   switch(RHS.aType)
   {
      case SCPTestDefinition::GREEDY     : os << "All Greedy " << endl; break;
      case SCPTestDefinition::GA         : os << "GA " << endl; break;
      case SCPTestDefinition::RANDOM_ALG : os << "Random " << endl; break;
      case SCPTestDefinition::BACK       : os << "Greedy with Back " << endl; break;
      case SCPTestDefinition::CHV        : os << "Greedy with Chavatal " << endl; break;
      case SCPTestDefinition::GRD3       : os << "Greedy with Greedy3 " << endl; break;
      case SCPTestDefinition::BCGA       : os << "Beasley&Chu  GA " << endl; break;
      case SCPTestDefinition::SSGA       : os << "Steady State GA " << endl; break;

   }// end of switch() 

   if( RHS.aType==SCPTestDefinition::GA || RHS.aType==SCPTestDefinition::BCGA ||
       RHS.aType==SCPTestDefinition::SSGA )
   {
      os << " Number of Generations .: " << RHS.numberOfGenerations << endl;
      os << " Size of Population ....: " << RHS.sizeOfPopulation << endl;
      os << " Crossover Operator ....: " ;

      switch(RHS.cType)
      {

         case SCPTestDefinition::RND1PX : os << "1PX with Random Restore " << endl; break;
         case SCPTestDefinition::CHV1PX : os << "1PX with Chavatal Restore " << endl; break;
         case SCPTestDefinition::BCK1PX : os << "1PX with Back Restore " << endl; break;
         case SCPTestDefinition::GR31PX : os << "1PX with Greedy3 Restore " << endl; break;

         case SCPTestDefinition::RND2PX : os << "2PX with Random Restore " << endl; break;
         case SCPTestDefinition::CHV2PX : os << "2PX with Chavatal Restore " << endl; break;
         case SCPTestDefinition::BCK2PX : os << "2PX with Back Restore " << endl; break;
         case SCPTestDefinition::GR32PX : os << "2PX with Greedy3 Restore " << endl; break;

         case SCPTestDefinition::UNIRND : os << "Uniform Crossover with Random Restore " << endl; break;
         case SCPTestDefinition::UNICHV : os << "Uniform Crossover with Chavatal Restore " << endl; break;
         case SCPTestDefinition::UNIBCK : os << "Uniform Crossover with Back Restore " << endl; break;
         case SCPTestDefinition::UNIGR3 : os << "Uniform Crossover with Greedy3 Restore " << endl; break;

         case SCPTestDefinition::FUSION : os << "Fusion Crossover with Back Restore " << endl; break;

      }// end of switch()

      os << " Crossover Probability .: " << RHS.cProb << endl;
      os << " Mutation  Operator ....: " ;

      switch(RHS.mType)
      {

         case SCPTestDefinition::DPXRND  : os << "DPX with Random Restore " << endl; break;
         case SCPTestDefinition::DPXCHV  : os << "DPX with Chavatal Restore " << endl; break;
         case SCPTestDefinition::DPXBCK  : os << "DPX with Back Restore " << endl; break;
         case SCPTestDefinition::DPXGR3  : os << "DPX with Greedy3 Restore " << endl; break;

      }// end of switch

      os << " Mutation Probability ..: " << RHS.mProb << endl;

   }// end of if

   return os;

}// end of operator<<()


